from functools import wraps

from sanic import Sanic, exceptions


class SanicTokenAuth:
    def __init__(self, app=None,
                 header=None,
                 token_verifier=None,
                 secret_key=None
                 ):
        self.secret_key = secret_key
        self.header = header
        self.token_verifier = token_verifier
        if app is not None:
            self.init_app(app)

    def init_app(self, app: Sanic):
        """hook on request start etc."""
        app.register_middleware(self.open_session, 'request')

    async def open_session(self, request):
        pass

    async def _is_authenticated(self, request):
        token = request.headers.get(self.header, None) if self.header else request.token
        print(token)
        if self.token_verifier:
            return await self.token_verifier(token)
        return token == self.secret_key

    def auth_required(self, handler=None):
        @wraps(handler)
        async def wrapper(request, *args, **kwargs):
            if not await self._is_authenticated(request):
                raise exceptions.Unauthorized("Auth required.")

            return await handler(request, *args, **kwargs)

        return wrapper



from sanic import Sanic
from sanic.response import text

from sanic_token_auth import SanicTokenAuth

app = Sanic()
auth = SanicTokenAuth(app, secret_key='utee3Quaaxohh1Oo', header='X-My-App-Auth-Token')


@app.route("/")
async def index(request):
    return text("Go to /protected")


@app.route("/protected")
@auth.auth_required
async def protected(request):
    return text("Welcome!")


if __name__ == "__main__":
    app.run(host="0.0.0.0", port=8000, debug=True)
